@@ -24,6 +24,8 @@
 #include "helpers/FilterListHelper.h"
 
 // List the global variables, proceeded by `extern`.
+extern bool globalChromiumInstalled;
 extern FilterListHelper *globalFilterListHelperPointer;
+extern bool globalFirefoxInstalled;
 
 #endif
