@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
 
 // Application headers.
 #include "DomainsDatabase.h"
+#include "Settings.h"
 #include "helpers/UserAgentHelper.h"
 
 // Define the private static schema constants.
-const int DomainsDatabase::SCHEMA_VERSION = 6;
+const int DomainsDatabase::SCHEMA_VERSION = 7;
 
 // Define the public static constants.
 const QString DomainsDatabase::CONNECTION_NAME = "domains_database";
@@ -30,9 +31,14 @@ const QString DomainsDatabase::CUSTOM_ZOOM_FACTOR = "custom_zoom_factor";
 const QString DomainsDatabase::DOM_STORAGE = "dom_storage";
 const QString DomainsDatabase::DOMAIN_NAME = "domain_name";
 const QString DomainsDatabase::DOMAINS_TABLE = "domains";
+const QString DomainsDatabase::EASYLIST = "easylist";
+const QString DomainsDatabase::EASYPRIVACY = "easyprivacy";
+const QString DomainsDatabase::FANBOYS_ANNOYANCE_LIST = "fanboys_annoyance_list";
 const QString DomainsDatabase::ID = "_id";
 const QString DomainsDatabase::JAVASCRIPT = "javascript";
 const QString DomainsDatabase::LOCAL_STORAGE = "local_storage";
+const QString DomainsDatabase::ULTRALIST = "ultralist";
+const QString DomainsDatabase::ULTRAPRIVACY = "ultraprivacy";
 const QString DomainsDatabase::USER_AGENT = "user_agent";
 const QString DomainsDatabase::ZOOM_FACTOR = "zoom_factor";
 
@@ -109,7 +115,7 @@ void DomainsDatabase::addDatabase()
                         [[fallthrough]];
                     }
 
-                    // Upgrade from schema version 4 to schema version 5.
+                    // Upgrade from schema version 4 to schema version 5, first used in Privacy Browser PC 0.1.
                     case 4:
                     {
                         // Add the Local Storage column.
@@ -119,7 +125,7 @@ void DomainsDatabase::addDatabase()
                         [[fallthrough]];
                     }
 
-                    // Upgrade from schema version 5 to schema version 6.
+                    // Upgrade from schema version 5 to schema version 6, first used in Privacy Browser PC 0.5.
                     case 5:
                     {
                         // Instantiate a spinner query.
@@ -228,6 +234,20 @@ void DomainsDatabase::addDatabase()
                             updateSpinnerQuery.exec();
                         }
 
+                        // Fall through to the next case.
+                        [[fallthrough]];
+                    }
+
+                    // Upgrade from schema version 6 to schema version 7, first used in Privacy Browser PC 0.6.
+                    case 6:
+                    {
+                        // Add the filter list columns.
+                        domainsDatabase.exec("ALTER TABLE " + DOMAINS_TABLE + " ADD COLUMN " + ULTRAPRIVACY + " INTEGER DEFAULT 0");
+                        domainsDatabase.exec("ALTER TABLE " + DOMAINS_TABLE + " ADD COLUMN " + ULTRALIST + " INTEGER DEFAULT 0");
+                        domainsDatabase.exec("ALTER TABLE " + DOMAINS_TABLE + " ADD COLUMN " + EASYPRIVACY + " INTEGER DEFAULT 0");
+                        domainsDatabase.exec("ALTER TABLE " + DOMAINS_TABLE + " ADD COLUMN " + EASYLIST + " INTEGER DEFAULT 0");
+                        domainsDatabase.exec("ALTER TABLE " + DOMAINS_TABLE + " ADD COLUMN " + FANBOYS_ANNOYANCE_LIST + " INTEGER DEFAULT 0");
+
                         // Fall through to the next case.
                         // [[fallthrough]];
                     }
@@ -250,6 +270,11 @@ void DomainsDatabase::addDatabase()
                                      LOCAL_STORAGE + " INTEGER DEFAULT 0, " +
                                      DOM_STORAGE + " INTEGER DEFAULT 0, " +
                                      USER_AGENT + " TEXT DEFAULT '" + UserAgentHelper::SYSTEM_DEFAULT_DATABASE + "', " +
+                                     ULTRAPRIVACY + " INTEGER DEFAULT 0, " +
+                                     ULTRALIST + " INTEGER DEFAULT 0, " +
+                                     EASYPRIVACY + " INTEGER DEFAULT 0, " +
+                                     EASYLIST + " INTEGER DEFAULT 0, " +
+                                     FANBOYS_ANNOYANCE_LIST + " INTEGER DEFAULT 0, " +
                                      ZOOM_FACTOR + " INTEGER DEFAULT 0, " +
                                      CUSTOM_ZOOM_FACTOR + " REAL DEFAULT 1.0)");
 
@@ -266,11 +291,63 @@ void DomainsDatabase::addDatabase()
     }
     else  // Opening the database failed.
     {
-        // Write the last database error message to the debug output.
+        // Write the last database error message to the debug output.Settings::zoom
         qDebug().noquote().nospace() << "Error opening database:  " << domainsDatabase.lastError();
     }
 };
 
+void DomainsDatabase::addDomain(const QString &domainName)
+{
+    // Add the domain:
+    addDomain(domainName, SYSTEM_DEFAULT, SYSTEM_DEFAULT, SYSTEM_DEFAULT, UserAgentHelper::SYSTEM_DEFAULT_DATABASE, SYSTEM_DEFAULT, SYSTEM_DEFAULT, SYSTEM_DEFAULT, SYSTEM_DEFAULT, SYSTEM_DEFAULT,
+              SYSTEM_DEFAULT, Settings::zoomFactor());
+}
+
+void DomainsDatabase::addDomain(const QString &domainName, const int javaScriptInt, const int localStorageInt, const int domStorageInt, const QString &userAgentDatabaseString,
+                                const int ultraPrivacyInt, const int ultraListInt, const int easyPrivacyInt, const int easyListInt, const int fanboysAnnoyanceListInt, const int zoomFactorInt,
+                                const double currentZoomFactorDouble)
+{
+    // Get a handle for the domains database.
+    QSqlDatabase domainsDatabase = QSqlDatabase::database(CONNECTION_NAME);
+
+    // Instantiate an add domain settings query.
+    QSqlQuery addDomainSettingsQuery(domainsDatabase);
+
+    // Prepare the query.
+    addDomainSettingsQuery.prepare("INSERT INTO " + DOMAINS_TABLE + " (" +
+                                    DOMAIN_NAME + ", " +
+                                    JAVASCRIPT + ", " +
+                                    LOCAL_STORAGE + ", " +
+                                    DOM_STORAGE + ", " +
+                                    USER_AGENT + ", " +
+                                    ULTRAPRIVACY + ", " +
+                                    ULTRALIST + ", " +
+                                    EASYPRIVACY + ", " +
+                                    EASYLIST + ", " +
+                                    FANBOYS_ANNOYANCE_LIST + ", " +
+                                    ZOOM_FACTOR + ", " +
+                                    CUSTOM_ZOOM_FACTOR + ") " +
+                                    "VALUES (:domain_name, :javascript, :local_storage, :dom_storage, :user_agent, :ultraprivacy, :ultralist, :easyprivacy, :easylist, :fanboys_annoyance_list, :zoom_factor, :custom_zoom_factor)"
+    );
+
+    // Bind the query values.
+    addDomainSettingsQuery.bindValue(":domain_name", domainName);
+    addDomainSettingsQuery.bindValue(":javascript", javaScriptInt);
+    addDomainSettingsQuery.bindValue(":local_storage", localStorageInt);
+    addDomainSettingsQuery.bindValue(":dom_storage", domStorageInt);
+    addDomainSettingsQuery.bindValue(":user_agent", userAgentDatabaseString);
+    addDomainSettingsQuery.bindValue(":ultraprivacy", ultraPrivacyInt);
+    addDomainSettingsQuery.bindValue(":ultralist", ultraListInt);
+    addDomainSettingsQuery.bindValue(":easyprivacy", easyPrivacyInt);
+    addDomainSettingsQuery.bindValue(":easylist", easyListInt);
+    addDomainSettingsQuery.bindValue(":fanboys_annoyance_list", fanboysAnnoyanceListInt);
+    addDomainSettingsQuery.bindValue(":zoom_factor", zoomFactorInt);
+    addDomainSettingsQuery.bindValue(":custom_zoom_factor", currentZoomFactorDouble);
+
+    // Execute the query.
+    addDomainSettingsQuery.exec();
+}
+
 QSqlQuery DomainsDatabase::getDomainQuery(const QString &hostname)
 {
     // Get a handle for the domains database.
