@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef DOMAINSDATABASE_H
-#define DOMAINSDATABASE_H
+#ifndef DOMAINS_DATABASE_H
+#define DOMAINS_DATABASE_H
 
 // Qt framework headers.
 #include <QtSql>
@@ -31,6 +31,9 @@ public:
 
     // The public functions.
     static void addDatabase();
+    static void addDomain(const QString &domainName);
+    static void addDomain(const QString &domainName, const int javaScriptInt, const int localStorageInt, const int domStorageInt, const QString &userAgentDatabaseString, const int ultraPrivacyInt,
+                          const int ultraListInt, const int easyPrivacyInt, const int easyListInt, const int fanboysAnnoyanceListInt, const int zoomFactorInt, const double currentZoomFactorDouble);
     static QSqlQuery getDomainQuery(const QString &hostname);
 
     // The public int constants.
@@ -45,9 +48,14 @@ public:
     static const QString DOM_STORAGE;
     static const QString DOMAIN_NAME;
     static const QString DOMAINS_TABLE;
+    static const QString EASYLIST;
+    static const QString EASYPRIVACY;
+    static const QString FANBOYS_ANNOYANCE_LIST;
     static const QString ID;
     static const QString JAVASCRIPT;
     static const QString LOCAL_STORAGE;
+    static const QString ULTRALIST;
+    static const QString ULTRAPRIVACY;
     static const QString USER_AGENT;
     static const QString ZOOM_FACTOR;
 
