@@ -1,5 +1,5 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
  * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
  *
@@ -50,11 +50,11 @@ private Q_SLOTS:
     void addCookieFromDialog(const QNetworkCookie &cookie, const bool &isDurable) const;
     void deleteCookieFromDatabase(const QNetworkCookie &cookie) const;
     void deleteCookieFromDialog(const QNetworkCookie &cookie) const;
-    void showAddCookieDialog() const;
+    void showAddCookieDialog();
     void showDeleteAllMessageBox() const;
     void showDeleteCookieMessageBox() const;
-    void showDurableCookiesDialog() const;
-    void showEditCookieDialog() const;
+    void showDurableCookiesDialog();
+    void showEditCookieDialog();
     void updateUi() const;
 
 private:
