@@ -1,5 +1,5 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
  * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
  *
@@ -26,7 +26,7 @@
 // KDE Frameworks headers.
 #include <KLocalizedString>
 
-DurableCookiesDialog::DurableCookiesDialog() : QDialog(nullptr)
+DurableCookiesDialog::DurableCookiesDialog(QWidget *parentWidgetPointer) : QDialog(parentWidgetPointer)
 {
     // Set the dialog window title.
     setWindowTitle(i18nc("The durable cookies dialog window title", "Durable Cookies"));
