@@ -18,7 +18,9 @@
  */
 
 // Application headers.
+#include "GlobalVariables.h"
 #include "PrivacyWebEngineView.h"
+#include "PrivacyWebEnginePage.h"
 #include "Settings.h"
 #include "ui_HttpAuthenticationDialog.h"
 #include "databases/CookiesDatabase.h"
@@ -39,13 +41,13 @@ PrivacyWebEngineView::PrivacyWebEngineView(QWidget *parentWidgetPointer) : QWebE
     webEngineProfilePointer = new QWebEngineProfile(QLatin1String(""));
 
     // Create a WebEngine page.
-    QWebEnginePage *webEnginePagePointer = new QWebEnginePage(webEngineProfilePointer);
+    PrivacyWebEnginePage *privacyWebEnginePagePointer = new PrivacyWebEnginePage(webEngineProfilePointer);
 
     // Set the WebEngine page.
-    setPage(webEnginePagePointer);
+    setPage(privacyWebEnginePagePointer);
 
     // Get handles for the various aspects of the WebEngine.
-    webEngineSettingsPointer = webEnginePagePointer->settings();
+    webEngineSettingsPointer = privacyWebEnginePagePointer->settings();
 
     // Instantiate the URL request interceptor.
     UrlRequestInterceptor *urlRequestInterceptorPointer = new UrlRequestInterceptor(this);
@@ -60,7 +62,10 @@ PrivacyWebEngineView::PrivacyWebEngineView(QWidget *parentWidgetPointer) : QWebE
     connect(urlRequestInterceptorPointer, SIGNAL(requestProcessed(RequestStruct*)), this, SLOT(storeRequest(RequestStruct*)));
 
     // Handle HTTP authentication requests.
-    connect(webEnginePagePointer, SIGNAL(authenticationRequired(const QUrl&, QAuthenticator*)), this, SLOT(handleAuthenticationRequest(const QUrl&, QAuthenticator*)));
+    connect(privacyWebEnginePagePointer, SIGNAL(authenticationRequired(const QUrl&, QAuthenticator*)), this, SLOT(handleAuthenticationRequest(const QUrl&, QAuthenticator*)));
+
+    // Store the link URL whenever a link is hovered.
+    connect(privacyWebEnginePagePointer, SIGNAL(linkHovered(const QString)), this, SLOT(saveHoveredLink(const QString)));
 }
 
 void PrivacyWebEngineView::addCookieToList(const QNetworkCookie &cookie) const
@@ -246,17 +251,50 @@ void PrivacyWebEngineView::contextMenuEvent(QContextMenuEvent *contextMenuEvent)
     // Add the open link in new background tab action if the context menu already contains the open link in new window action.
     if (contextMenuActionsList.contains(webEnginePagePointer->action(QWebEnginePage::OpenLinkInNewWindow)))
     {
-        // Move the open in new tab action to the top of the list.
+        // Move the open in new tab action above the back action.
         contextMenu->insertAction(webEnginePagePointer->action(QWebEnginePage::Back), webEnginePagePointer->action(QWebEnginePage::OpenLinkInNewTab));
 
-        // Add the open link in background tab action below the open in new tab action.
+        // Add the open link in background tab action above the back action.
         contextMenu->insertAction(webEnginePagePointer->action(QWebEnginePage::Back), webEnginePagePointer->action(QWebEnginePage::OpenLinkInNewBackgroundTab));
 
-        // Move the open in new window action below the open in background tab action.
+        // Move the open in new window action above the back action.
         contextMenu->insertAction(webEnginePagePointer->action(QWebEnginePage::Back), webEnginePagePointer->action(QWebEnginePage::OpenLinkInNewWindow));
 
-        // Add a separator below the open in new window action.
+        // Add a separator above the back action.
         contextMenu->insertSeparator(webEnginePagePointer->action(QWebEnginePage::Back));
+
+        if (globalFirefoxInstalled || globalChromiumInstalled)
+        {
+            // Add the open with Firefox action if Firefox is installed.
+            if (globalFirefoxInstalled)
+            {
+                // Create an open with Firefox action.
+                QAction *openWithFirefoxActionPointer = new QAction(QIcon::fromTheme(QLatin1String("firefox-esr")), i18nc("Open with Firefox context menu action", "Open with Firefox"), contextMenu);
+
+                // Add the open with Firefox action above the back action.
+                contextMenu->insertAction(webEnginePagePointer->action(QWebEnginePage::Back), openWithFirefoxActionPointer);
+
+                // Connect the action.
+                connect(openWithFirefoxActionPointer, SIGNAL(triggered()), this, SLOT(openWithFirefox()));
+            }
+
+            // Add the open with Chromium action if Chromium is installed.
+            if (globalChromiumInstalled)
+            {
+                // Create an open with Chromium action.
+                QAction *openWithChromiumActionPointer = new QAction(QIcon::fromTheme(QLatin1String("chromium")), i18nc("Open with Chromium context menu action", "Open with Chromium"), contextMenu);
+
+                // Add the open with Chromium action above the back action.
+                contextMenu->insertAction(webEnginePagePointer->action(QWebEnginePage::Back), openWithChromiumActionPointer);
+
+                // Connect the action.
+                connect(openWithChromiumActionPointer, SIGNAL(triggered()), this, SLOT(openWithChromium()));
+            }
+
+
+            // Add a separator above the back action.
+            contextMenu->insertSeparator(webEnginePagePointer->action(QWebEnginePage::Back));
+        }
     }
 
     // Display the menu using the location in the context menu event.
@@ -326,6 +364,18 @@ void PrivacyWebEngineView::handleAuthenticationRequest(const QUrl &requestUrl, Q
     }
 }
 
+void PrivacyWebEngineView::openWithChromium() const
+{
+    // Open the current URL in Chromium
+    QProcess::startDetached("chromium", QStringList(hoveredLinkString));
+}
+
+void PrivacyWebEngineView::openWithFirefox() const
+{
+    // Open the current URL in Firefox.
+    QProcess::startDetached("firefox-esr", QStringList(hoveredLinkString));
+}
+
 void PrivacyWebEngineView::removeCookieFromList(const QNetworkCookie &cookie) const
 {
     //qDebug() << "Remove cookie:  " << cookie.toRawForm();
@@ -337,6 +387,12 @@ void PrivacyWebEngineView::removeCookieFromList(const QNetworkCookie &cookie) co
     emit numberOfCookiesChanged(cookieListPointer->size());
 }
 
+void PrivacyWebEngineView::saveHoveredLink(const QString &hoveredLink)
+{
+    // Save the hovered link.
+    hoveredLinkString = hoveredLink;
+}
+
 void PrivacyWebEngineView::storeRequest(RequestStruct *requestStructPointer)
 {
     // Store the request struct in the list.
