@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TABWIDGET_H
-#define TABWIDGET_H
+#ifndef TAB_WIDGET_H
+#define TAB_WIDGET_H
 
 // Application headers.
 #include "PrivacyWebEngineView.h"
@@ -50,9 +50,11 @@ public:
     // The destructor.
     ~TabWidget();
 
+    // The public variables.
+    PrivacyWebEngineView *currentPrivacyWebEngineViewPointer;
+
     // The public functions.
-    void applyOnTheFlyZoomFactor(const double &zoomFactor) const;
-    void applySpellCheckLanguages() const;
+    void applyOnTheFlyZoomFactor(const double zoomFactorDouble) const;
     PrivacyWebEngineView* loadBlankInitialWebsite();
     void loadInitialWebsite();
     void findPrevious(const QString &text) const;
@@ -60,8 +62,10 @@ public:
     QIcon getCurrentTabFavoritIcon() const;
     QString getCurrentTabTitle() const;
     QString getCurrentTabUrl() const;
+    QString getCurrentUserAgent() const;
     QString& getDomainSettingsName() const;
     void setTabBarVisible(const bool visible) const;
+    void toggleDeveloperTools(const bool enabled) const;
     void toggleDomStorage() const;
     void toggleFindCaseSensitive(const QString &text);
     void toggleJavaScript() const;
@@ -73,36 +77,44 @@ public:
 signals:
     // The signals.
     void addCookie(const QNetworkCookie &cookie) const;
+    void blockedRequestsUpdated(const QVector<int> blockedRequests) const;
+    void cookiesChanged(const int numberOfCookies) const;
+    void easyListStatusChanged(const bool isEnabled) const;
+    void easyPrivacyStatusChanged(const bool isEnabled) const;
+    void fanboysAnnoyanceListStatusChanged(const bool isEnabled) const;
     void removeCookie(const QNetworkCookie &cookie) const;
     void clearUrlLineEditFocus() const;
     void fullScreenRequested(const bool toggleOn) const;
     void hideProgressBar() const;
     void linkHovered(const QString &linkUrl) const;
     void showProgressBar(const int &progress) const;
-    void updateBackAction(const bool &isEnabled) const;
-    void updateCookiesAction(const int numberOfCookies) const;
-    void updateDefaultZoomFactor(const double newDefaultZoomFactor) const;
-    void updateDomStorageAction(const bool &isEnabled) const;
+    void ultraListStatusChanged(const bool) const;
+    void ultraPrivacyStatusChanged(const bool) const;
+    void updateBackAction(const bool isEnabled) const;
+    void updateDefaultZoomFactor(const double newDefaultZoomFactorDouble) const;
+    void updateDeveloperToolsAction(const bool isEnabled) const;
+    void updateDomStorageAction(const bool isEnabled) const;
     void updateDomainSettingsIndicator(const bool status) const;
     void updateFindText(const QString &text, const bool findCaseSensitive) const;
     void updateFindTextResults(const QWebEngineFindTextResult &findTextResult) const;
-    void updateForwardAction(const bool &isEnabled) const;
-    void updateJavaScriptAction(const bool &isEnabled) const;
-    void updateLocalStorageAction(const bool &isEnabled) const;
+    void updateForwardAction(const bool isEnabled) const;
+    void updateJavaScriptAction(const bool isEnabled) const;
+    void updateLocalStorageAction(const bool isEnabled) const;
     void updateSearchEngineActions(const QString &searchEngine, const bool &updateCustomSearchEngineStatus) const;
     void updateUrlLineEdit(const QUrl &newUrl) const;
     void updateUserAgentActions(const QString &userAgent, const bool &updateCustomUserAgentStatus) const;
     void updateWindowTitle(const QString &title) const;
-    void updateZoomActions(const double &zoomFactor) const;
+    void updateZoomActions(const double zoomFactorDouble) const;
 
 public Q_SLOTS:
     // The public slots.
     void addCookieToStore(QNetworkCookie cookie, QWebEngineCookieStore *webEngineCookieStorePointer = nullptr) const;
-    PrivacyWebEngineView* addTab(const bool removeUrlLineEditFocus = false, const bool backgroundTab = false, const QString urlString = nullptr);
+    PrivacyWebEngineView* addTab(const bool removeUrlLineEditFocus = false, const bool adjacent = false, const bool backgroundTab = false, const QString urlString = nullptr);
     void applyApplicationSettings();
     void applyDomainSettingsAndReload();
     void applyOnTheFlySearchEngine(QAction *searchEngineActionPointer);
     void applyOnTheFlyUserAgent(QAction *userAgentActionPointer) const;
+    void applySpellCheckLanguages() const;
     void back() const;
     void deleteAllCookies() const;
     void deleteCookieFromStore(const QNetworkCookie &cookie) const;
@@ -116,6 +128,13 @@ public Q_SLOTS:
     void printPreview() const;
     void refresh() const;
     void reloadAndBypassCache() const;
+    void saveArchive();
+    void stop() const;
+    void toggleEasyList() const;
+    void toggleEasyPrivacy() const;
+    void toggleFanboysAnnoyanceList() const;
+    void toggleUltraList() const;
+    void toggleUltraPrivacy() const;
 
 private Q_SLOTS:
     // The private slots.
@@ -132,8 +151,10 @@ private Q_SLOTS:
     void useNativeKdeDownloader(QUrl &downloadUrl, QString &suggestedFileName);
 
 private:
+    // The private functions.
+    void updateDownloadDirectory(QString newDownloadDirectory) const;
+
     // The private variables.
-    PrivacyWebEngineView *currentPrivacyWebEngineViewPointer;
     QWebEngineCookieStore *currentWebEngineCookieStorePointer;
     QWebEngineHistory *currentWebEngineHistoryPointer;
     QWebEnginePage *currentWebEnginePagePointer;
@@ -142,8 +163,9 @@ private:
     QIcon defaultFavoriteIcon = QIcon::fromTheme(QLatin1String("globe"), QIcon::fromTheme(QLatin1String("applications-internet")));
     bool isRunningKde = false;
     QMovie *loadingFavoriteIconMoviePointer;
-    QString searchEngineUrl;
     QTabWidget *qTabWidgetPointer;
+    bool savingArchive;
+    QString searchEngineUrl;
     UserAgentHelper *userAgentHelperPointer;
     bool wipingCurrentFindTextSelection = false;
 };
